// • ▌ ▄ ·.  ▄▄▄·  ▄▄ • ▪   ▄▄· ▄▄▄▄·  ▄▄▄·  ▐▄▄▄  ▄▄▄ .
// ·██ ▐███▪▐█ ▀█ ▐█ ▀ ▪██ ▐█ ▌▪▐█ ▀█▪▐█ ▀█ •█▌ ▐█▐▌·
// ▐█ ▌▐▌▐█·▄█▀▀█ ▄█ ▀█▄▐█·██ ▄▄▐█▀▀█▄▄█▀▀█ ▐█▐ ▐▌▐▀▀▀
// ██ ██▌▐█▌▐█ ▪▐▌▐█▄▪▐█▐█▌▐███▌██▄▪▐█▐█ ▪▐▌██▐ █▌▐█▄▄▌
// ▀▀  █▪▀▀▀ ▀  ▀ ·▀▀▀▀ ▀▀▀·▀▀▀ ·▀▀▀▀  ▀  ▀ ▀▀  █▪ ▀▀▀
//      Magicbane Emulator Project © 2013 - 2022
//                www.magicbane.com


package engine.net.client.msg;

import engine.Enum.GameObjectType;
import engine.exception.SerializationException;
import engine.net.AbstractConnection;
import engine.net.ByteBufferReader;
import engine.net.ByteBufferWriter;
import engine.net.client.Protocol;
import engine.objects.PlayerCharacter;


public class RecommendNationMsg extends ClientNetMsg {


    private int guildID;
    private byte ally;
    private byte enemy;


    public RecommendNationMsg(PlayerCharacter player) {
        super(Protocol.RECOMMENDNATION);

    }

    public RecommendNationMsg() {
        super(Protocol.RECOMMENDNATION);
    }


    /**
     * This constructor is used by NetMsgFactory. It attempts to deserialize the
     * ByteBuffer into a message. If a BufferUnderflow occurs (based on reading
     * past the limit) then this constructor Throws that Exception to the
     * caller.
     */
    public RecommendNationMsg(AbstractConnection origin, ByteBufferReader reader) {
        super(Protocol.RECOMMENDNATION, origin, reader);
    }
    //CALL THIS AFTER SANITY CHECKS AND BEFORE UPDATING HEALTH/GOLD.


    /**
     * Deserializes the subclass specific items from the supplied NetMsgReader.
     */
    @Override
    protected void _deserialize(ByteBufferReader reader) {
        reader.getInt();
        this.guildID = reader.getInt();
        this.ally = reader.get();
        this.enemy = reader.get();
    }


    // Precache and configure this message before we serialize it


    /**
     * Serializes the subclass specific items to the supplied NetMsgWriter.
     */
    @Override
    protected void _serialize(ByteBufferWriter writer) throws SerializationException {
        writer.putInt(GameObjectType.Guild.ordinal());
        writer.putInt(guildID);
        writer.put(this.ally);
        writer.put(this.enemy);
    }

    public int getGuildID() {
        return guildID;
    }

    public byte getAlly() {
        return ally;
    }

    public byte getEnemy() {
        return enemy;
    }

}
